{% extends "login_email/base.html" %}

{% block content %}
    <h3>login with email</h3>
    <form action="" method='post'>
        {% csrf_token %}
        {{ form.as_p }}
        <button type="submit">submit</button>
    </form>
{% endblock content %}